<?php
/**
 * Upgrade Functions
 *
 * @package     EDD
 * @subpackage  Admin/Upgrades
 * @copyright   Copyright (c) 2015, Pippin Williamson
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       2.4
 */


/**
 * Recurring Payments Upgrade Notices
 *
 * @since 2.4
 *
 */
function edd_show_recurring_upgrade_notices() {

	global $wpdb;

	// Don't show notices on the upgrades page
	if ( isset( $_GET['page'] ) && $_GET['page'] == 'edd-upgrades' ) {
		return;
	}

	$edd_recurring_version = get_option( 'edd_recurring_version' );

	if ( ! edd_has_upgrade_completed( 'upgrade_24_subscriptions' ) ) {

		$has_recurring = $wpdb->get_var( "SELECT post_id FROM $wpdb->postmeta WHERE meta_key = 'edd_period' OR ( meta_key = 'edd_variable_prices' AND meta_value LIKE '%recurring%' AND meta_value LIKE '%yes%' ) AND 1=1 LIMIT 1" );
		$needs_upgrade = ! empty( $has_recurring );

		if( ! $needs_upgrade ) {
			return;
		}

		printf(
			'<div class="updated"><p>' . __( 'Easy Digital Downloads needs to upgrade the subscriptions database, click <a href="%s">here</a> to start the upgrade.', 'edd-recurring' ) . '</p></div>',
			esc_url( admin_url( 'index.php?page=edd-upgrades&edd-upgrade=upgrade_24_subscriptions' ) )
		);
	}

}
add_action( 'admin_notices', 'edd_show_recurring_upgrade_notices' );

/**
 * Migrates pre 2.4 subscriptions to new database
 *
 * @since  2.4
 * @return void
 */
function edd_recurring_v24_migrate_subscriptions() {
	global $wpdb;

	if ( ! current_user_can( 'manage_shop_settings' ) ) {
		wp_die( __( 'You do not have permission to do shop upgrades', 'edd' ), __( 'Error', 'edd' ), array( 'response' => 403 ) );
	}

	ignore_user_abort( true );
	if ( ! edd_is_func_disabled( 'set_time_limit' ) && ! ini_get( 'safe_mode' ) ) {
		@set_time_limit( 0 );
	}

	$step   = isset( $_GET['step'] ) ? absint( $_GET['step'] ) : 1;
	$number = isset( $_GET['number'] ) ? absint( $number ) : 5;
	$offset = $step == 1 ? 0 : ( $step - 1 ) * $number;

	if ( $step < 2 ) {

		$db = new EDD_Subscriptions_DB;
		@$db->create_table();

		// Check if we have any payments before moving on
		$sql          = "SELECT ID FROM $wpdb->posts WHERE post_type = 'edd_payment' LIMIT 1";
		$has_payments = $wpdb->get_col( $sql );

		if ( empty( $has_payments ) ) {
			// We had no payments, just complete
			update_option( 'edd_recurring_version', preg_replace( '/[^0-9.].*/', '', EDD_RECURRING_VERSION ) );
			edd_set_upgrade_complete( 'upgrade_24_subscriptions' );
			delete_option( 'edd_doing_upgrade' );
			wp_redirect( admin_url() );
			exit;
		}
	}

	$total = isset( $_GET['total'] ) ? absint( $_GET['total'] ) : false;
	if ( empty( $total ) || $total <= 1 ) {
		$total_sql = "SELECT COUNT(ID) as total_payments FROM $wpdb->posts WHERE post_type = 'edd_payment' AND post_status IN ('publish','revoked','cancelled');";
		$results   = $wpdb->get_row( $total_sql, 0 );
		$total     = $results->total_payments;
	}

	$payment_ids = $wpdb->get_col( $wpdb->prepare( "SELECT ID FROM $wpdb->posts WHERE post_type = 'edd_payment' AND post_status IN ('publish','revoked','cancelled') ORDER BY post_date ASC LIMIT %d,%d;", $offset, $number ) );

	if ( $payment_ids ) {

		foreach ( $payment_ids as $payment_id ) {

			$cancelled    = false;
			$expiration   = '';
			$payment      = new EDD_Payment( $payment_id );
			$profile      = edd_recurring_get_legacy_profile_id( $payment );
			$cart_details = $payment->cart_details;
			$download     = reset( $cart_details );
			$download_id  = $download['id'];

			if( is_object( $profile ) ) {

				// Stripe gives us a subscription object containing the sub ID, customer ID, and cancelled status

				$profile_id   = $profile->id;
				$recurring_id = $profile->customer;
				$cancelled    = ! empty( $profile->canceled_at );
				$expiration   = $profile->current_period_end;

			} else {

				$profile_id   = $profile;

			}

			if ( edd_has_variable_prices( $download_id ) ) {

				$price_id  = edd_get_cart_item_price_id( $download );
				$recurring = edd_recurring()->is_price_recurring( $download_id, $price_id );
				$times     = edd_recurring()->get_times( $price_id, $download_id );
				$period    = edd_recurring()->get_period( $price_id, $download_id );
				$fee       = edd_recurring()->get_signup_fee( $price_id, $download_id );

			} else {

				$recurring = edd_recurring()->is_recurring( $download_id );
				$times     = edd_recurring()->get_times_single( $download_id );
				$period    = edd_recurring()->get_period_single( $download_id );
				$fee       = edd_recurring()->get_signup_fee_single( $download_id );

			}

			if( ! $recurring ) {
				continue;
			}

			if( empty( $payment->user_id ) ) {

				$user = get_user_by( 'email', $payment->email );

				if( $user ) {

					$payment->user_id = $user->ID;

				} else {

					remove_all_actions( 'user_register' );

					$payment->user_id = wp_insert_user( array(
						'user_email' => $payment->email,
						'user_login' => $payment->email,
						'first_name' => $payment->first_name,
						'last_name'  => $payment->last_name,
					) );

				}

				$payment->save();

			}

			$subscriber = new EDD_Recurring_Subscriber( $payment->user_id, true );

			if( ! empty( $recurring_id ) ) {
				$subscriber->set_recurring_customer_id( $recurring_id, 'stripe' );
			}

			if( empty( $expiration ) ) {

				$expiration  = get_user_meta( $payment->user_id, '_edd_recurring_exp', true );

				if( empty( $expiration ) ) {

					// If no user account existed, we don't know the expiration date

					$child_payments = get_posts( array(
						'post_type'      => 'edd_payment',
						'post_status'    => 'edd_subscription',
						'posts_per_page' => 1,
						'post_parent'    => $payment_id,
						'order'          => 'DESC',
						'orderby'        => 'post_date'
					) );

					if( $child_payments ) {

						// Use date of latest renewal payment as base
						$child_payment = reset( $child_payments );
						$base = strtotime( $child_payment->post_date );

					} else {

						// Use signup date as base if there are no renewals
						$base = strtotime( $payment->date );

					}

					$expiration = strtotime( '+ 1 ' . $period . ' 23:59:59', $base );

				}

			}

			switch( $payment->status ) {

				case 'publish' :

					$status = 'active';
					break;

				case 'cancelled' :
				case 'revoked' :

					$status = 'cancelled';
					break;
			}

			if( ! empty( $cancelled ) ) {
				$status = 'cancelled';
			}

			$args = array(
				'product_id'        => $download_id,
				'period'            => $period,
				'initial_amount'    => $payment->total,
				'recurring_amount'  => $payment->total + $fee,
				'bill_times'        => $times,
				'parent_payment_id' => $payment_id,
				'created'           => $payment->date,
				'gateway'           => $payment->gateway,
				'expiration'        => date( 'Y-m-d H:i:s', $expiration ),
				'profile_id'        => $profile_id,
				'status'            => $status,
			);

			$subscriber->add_subscription( $args );

			edd_update_payment_meta( $payment_id, '_edd_subscription_payment', true );

		}

		$step ++;
		$redirect = add_query_arg( array(
			'page'        => 'edd-upgrades',
			'edd-upgrade' => 'upgrade_24_subscriptions',
			'step'        => $step,
			'number'      => $number,
			'total'       => $total
		), admin_url( 'index.php' ) );

		wp_redirect( $redirect );
		exit;

	} else {

		update_option( 'edd_recurring_version', preg_replace( '/[^0-9.].*/', '', EDD_RECURRING_VERSION ) );
		edd_set_upgrade_complete( 'upgrade_24_subscriptions' );
		delete_option( 'edd_doing_upgrade' );

		wp_redirect( admin_url() );
		exit;

	}

}
add_action( 'edd_upgrade_24_subscriptions', 'edd_recurring_v24_migrate_subscriptions' );

/**
 * Retrieve the recurring profile ID for a legacy subscription
 *
 * @since  2.4
 * @return string
 */
function edd_recurring_get_legacy_profile_id( EDD_Payment $payment ) {

	$profile_id = '';

	switch( $payment->gateway ) {

		case 'paypal' :

			foreach ( edd_get_payment_notes( $payment->ID ) as $note ) {
				if ( preg_match( '/^PayPal Subscription ID: ([^\s]+)/', $note->comment_content, $match ) ) {
					$profile_id = $match[1];
					continue;
				}
			}

			break;

		case 'stripe' :

			$charge_id = $payment->transaction_id;

			if( empty( $charge_id ) ) {

				if ( edd_is_test_mode() ) {
					$prefix = 'test_';
				} else {
					$prefix = 'live_';
				}

				$secret_key = edd_get_option( $prefix . 'secret_key', '' );

				\Stripe\Stripe::setApiKey( $secret_key );

				foreach ( edd_get_payment_notes( $payment->ID ) as $note ) {
					if ( preg_match( '/^Stripe Charge ID: ([^\s]+)/', $note->comment_content, $match ) ) {
						$charge_id = $match[1];

						// We need to store the transaction ID if it's not present
						$payment->transaction_id = $charge_id;
						$payment->save();

						continue;
					}
					if ( preg_match( '/^Stripe Customer ID: ([^\s]+)/', $note->comment_content, $match ) ) {
						$customer_id = $match[1];
						continue;
					}
				}

				if( empty( $charge_id ) ) {

					// No charge, let's look for a customer ID
					if( ! empty( $customer_id ) ) {

						try {

							$customer      = \Stripe\Customer::retrieve( $customer_id );
							$subscriptions = $customer->subscriptions->data;
							$subscription  = reset( $subscriptions );

							if( ! empty( $subscription ) ) {
								return $subscription;
							}

							return '';

						} catch ( Exception $e ) {

							return '';

						}

					}

				}

			}

			try {

				$charge     = \Stripe\Charge::retrieve( $charge_id );
				$invoice    = \Stripe\Invoice::retrieve( $charge->invoice );
				$customer   = \Stripe\Customer::retrieve( $invoice->customer );
				$profile_id = ! empty( $invoice->subscription ) ? $invoice->subscription : '';

				if( is_object( $customer ) && true !== $customer->deleted && ! empty( $profile_id ) ) {

					return $customer->subscriptions->retrieve( $profile_id );

				}

			} catch ( Exception $e ) {

				return '';

			}

			break;


	}

	return $profile_id;
}
